# -*- coding: utf-8 -*-
"""
Created on Fri Apr 23 10:23:38 2021

@author: Lino
"""
import argparse
import random 

#in_file=sys.argv[1]

def main(args):
    in_file=args.file
    with open (in_file) as f:
        string=f.read()
    
    string=string.rstrip()
    string=list(string)
    
    for i in range(len(string)+1,0,-1):
        j=random(0,len(string)+1)
        string[i],string[j] = string[j],string[i]



if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='')
    parser.add_argument("file", help="pah to input file ")
    args=parser.parse_args()
    main(args)


# -*- coding: utf-8 -*-
"""
Created on Fri Apr 23 19:57:17 2021

@author: Lino
"""

import argparse
from random import sample



#k_length=2
"""
###################################### hard coded!!!!!!! ###########
file_in="KLetShuffle-test30_k2.in"

k_length=3

#########################!!!!!!
#"""




class Graph ():
    
    def __init__(self,graph_dict=None):
        if graph_dict == None :
            graph_dict = {}
        self.graph_dict = graph_dict
        self.path=[]    
    
    # constructing the graph
    def add_vertex(self,vertex):
        if vertex not in self.graph_dict:
            self.graph_dict[vertex]=[]
    
    def add_edge (self,edge):
        (vertex_1,vertex_2)= tuple(edge)
        if len(vertex_1) != len(vertex_2):
            return
        if vertex_1 not in self.graph_dict:
            self.graph_dict[vertex_1]=vertex_2
        else:
            self.graph_dict[vertex_1].append(vertex_2)
    
    #constructing string from path 
    def path_to_string(self):
        k1,path=self.path[0],self.path[1:]
        for i in path:
            k1+=i[-1]
        string="".join(k1)
        return string
    
    # doing the recursion and back tracking to find the path 
    
    def euler (self,vertex):
        neigh_bour=self.graph_dict[vertex]
        permuated_vertices=sample(neigh_bour,len(neigh_bour))
        
        if len(self.path) == 0:

            self.path.append(vertex)
       # print(f"curent node {vertex} ,{permuated_vertices} ")
        if len(permuated_vertices)!= 0:
            for random_v in permuated_vertices:
                    
                    #print(f"in loop: curent node {vertex},next{random_v} ,{permuated_vertices} ")
                    
                    if len(self.path) < len(seq):
                        neigh_bour.remove(random_v)
                    
                    self.path.append(random_v)
                                        
                    token=self.euler(random_v)
                    
                    if token == "$back":
                        
                        edge_back=list([vertex,self.path.pop()])
                        self.add_edge(edge_back)
                        #print(f"here next node {random_v},current:{vertex},reurn{edge_back}")
                    else:
                        if token == "$fin":                            
                           
                            
                            return "$fin"
                        else:
                            continue
            return "$back"
        else:
            if len(self.path) == len(seq)-(k_length-2):
                #print("fin!!!")
                return "$fin"
            else:
               # print("sackgasse")

                return "$back"
                
        return 

############################################################
## mainly used for diagnostic purposses, not used in alg ##
############################################################
    
    def edges (self):
        edges = []
        for v in self.graph_dict:
            for neighbour in self.graph_dict[v]:
                     edges.append((v, neighbour))
        return edges
    
    def print_dict (self):
        print(self.graph_dict)
    
    def vertices (self):
        return list(self.graph_dict.keys())
    
    def path_list(self):
        #print(self.path)
        return(self.path)
            

def main(args):
    file_in=args.file
    global k_length
    k_length= int(args.klet)
    
    global seq
    with open (file_in) as f:
        seq=f.read()
    seq=seq.rstrip()
    
    ##########################
    ### constructing graph ###
    ##########################
    
    
    
    obj=Graph()
    
    for i in range(len(seq)-1):
        k1_let=seq[i:i+k_length-1]
        #print(k1_let)
        k1_let_conseq=seq[i+1:i+k_length]
        obj.add_vertex(k1_let)
        edge=[k1_let,k1_let_conseq]
        obj.add_edge(edge)
    
    
    ########################################################
    ### calling graph class to generat the shuffeld string##
    ########################################################
    
    
    #for random start vertices / does probably not work 
    #start_v=sample(obj.vertices(),1)[0]
    #something=obj.euler(start_v)
    
    find_path=obj.euler(seq[0:k_length-1])
    
    #print(start_v)
    """
    print(obj.print_dict())
    print(obj.path_list())
    print(len(obj.path_list()))
    #"""
    
    shuffeld_string=obj.path_to_string()
    
    print(f"\n input sequence: \n\t{seq}\n\n the shuffeled sequence: \n\t{shuffeld_string}\n")
    
    return


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='')
    parser.add_argument("file", help="pah to input file ")
    
    parser.add_argument("-k","--klet",
                        type=int,
                        help="k-let length")
    args=parser.parse_args()
    main(args)
    




# -*- coding: utf-8 -*-
"""
Created on Fri Apr  2 12:23:01 2021

@author: Lino Boehler
"""
import numpy as np
from collections import deque




def manhatten (list_weigths,diagonal,matrix,n,best_path):
    
    # numpy arrays holding the edge weigths 
    nord_south=np.array(list_weigths[0])
    
    west_east=np.array(list_weigths[1]) 
    
    if diagonal == True :
        diagonal_weight=np.array(list_weigths[2])
    
    # matrix with the longest paths to each node 
    pos_matrix=np.empty(shape=(n,n),dtype=object)
    
    # a list holding the path with directional strings :"E", "S" and ev. "D"
    # using deque data structure from collections module instead of list, since more convinient adn efficent.
    # appending from the left in deque lies in O(1) for a list it is O(n)
    longest_path=deque([])
    
    
    # calcluating the longest paths for the first column ...
    for i in range(1,n):
        matrix[i,0]=nord_south[i-1,0]+matrix[i-1,0]
        pos_matrix[i,0]="S"
    #... and firts row
    for j in range (1,n):
        matrix[0,j]= west_east[0,j-1]+matrix[0,j-1]
        pos_matrix[0,j]="E"
        
    
    # going over each matrix element row by row 
    for i in range(1,n):
        
        for j in range(1,n):
            
            
            # if total path length from above longer then total path length from left matrix entry 
            if matrix[i-1,j]+ nord_south[i-1,j] >= matrix[i,j-1]+ west_east[i,j-1]:
                
               
                if diagonal == True:
                     # if diagonal movements allowed check if diagonal path is longer then the path from above (north)
                    if  matrix[i-1,j-1] + diagonal_weight[i-1,j-1] >= matrix[i-1,j] + nord_south[i-1,j] :
                        matrix[i,j]= matrix[i-1,j-1] + diagonal_weight[i-1,j-1]
                        pos_matrix[i,j]="D"
                    else:
                        matrix[i,j]= nord_south[i-1,j] + matrix[i-1,j]
                        pos_matrix[i,j]="S"
                
                # if path from north is longer or diagnola moves are not allowed
                #then add maximal path length into the matrix of the curent position 
                # and enter the direction from where the longest path to curent postion into the matrix 
                #pos_matrix at the indices of curent position 
                else:
                    matrix[i,j]= nord_south[i-1,j] + matrix[i-1,j]
                    pos_matrix[i,j]="S"
                
            # if the total path length from east (left matrix entry ) is longer 
            else:
                
                if diagonal == True:
                    if  matrix[i-1,j-1] + diagonal_weight[i-1,j-1] >= matrix[i,j-1]+ west_east[i,j-1]:
                        matrix[i,j]= matrix[i-1,j-1] + diagonal_weight[i-1,j-1]
                        pos_matrix[i,j]="D"
                    else:
                        matrix[i,j]= matrix[i,j-1]+ west_east[i,j-1]
                        pos_matrix[i,j]="E"
                    
                else:
                    matrix[i,j]= matrix[i,j-1]+ west_east[i,j-1]
                    pos_matrix[i,j]="E"
   
    # back tracking over the "pos_matrix " thus going back along the matrix according to the directionla
    #informtion stored in the matrix entries 
    if best_path == True:
        i=n-1
        j=n-1
        while i or j !=0 :
            longest_path.appendleft(pos_matrix[i,j])
                            
            if pos_matrix[i,j] == "S":
                i-=1
            elif pos_matrix[i,j]== "E":
                j-=1
            else :
                i-=1
                j-=1
        print(f"the longest path is: {longest_path}")
    print(f"path length: {round(matrix[n-1,n-1],2)}")
    #print(matrix)
    return

        
    
def main(args):
    path_file=args.file
    diagonal=args.diagonal
    best_path=args.path
    
    list_south=[]
    list_east=[]
    list_weigths=[list_south,list_east]
    if diagonal == True:
        list_diag=[]
        list_weigths.append(list_diag)
    
    with open (path_file) as f:
       lines=f.readlines()
       counter=0
       ind=0
       n=0
       row_counter=0
       for l in lines:
            #skiping empyt ines and lines starting with # comments
            if l[:1] == "#" or l == "\n":
                
                continue
           
            else:
                l=l.rstrip()
                l=l.split()
                l=[float(i) for i in l]
                for i in l:
                    if i < 1:
                        counter+=1
                
                
                if len(list_weigths[0])==0:
                    n=len(l)
                
                if len(l)< n:
                    ind=1
                    row_counter+=1
                    
                    if row_counter > n:
                        if diagonal == True:
                            ind=2
                        else:
                            break
                
                list_weigths[ind].append(l)
       
       matrix = np.zeros(shape=(n,n),dtype=float)

       manhatten(list_weigths,diagonal,matrix,n,best_path)


if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description='')

    parser.add_argument("file", help="pah to input file ")

    parser.add_argument("-d", "--diagonal",
                    help="use this flage if the input file contains diagonal weights",
                         action="store_true")

    parser.add_argument("-t", "--path",
                    help="prints the best path if set true",
                    action="store_true")

    args = parser.parse_args()
    
    main(args)


# -*- coding: utf-8 -*-
"""
Created on Thu Apr 22 20:30:46 2021

@author: Lino Boehler
"""
import sys
from collections import Counter
from numpy import random as rd


in_file=sys.argv[1]
Nr_strg=int(sys.argv[2])

main(args):
	Nr_strg=args.Nstr
	in_file=args.file
	
	with open (in_file) as f:
		  string=f.read()

	string=string.rstrip()

	string=list(string)
	char_set=frozenset(string)
	char_set=list(char_set)
	char_count=Counter(string)

	char_prob=[]
	for i in char_set:
		  char_count[i]= char_count[i]/len(string)
		  char_prob.append(char_count[i])

	new_strings=rd.choice(char_set, (Nr_strg,len(string)), p=char_prob)
	return
	
if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='')
    parser.add_argument("file", help="pah to input file ")
    
    parser.add_argument("-nr","--Nstr",
                        help="number of strings")
    args=parser.parse_args()
    main(args)



#!/usr/bin/env python3
"""
Created on Sun Mar  7 22:18:11 2021

@author: Lino
"""
import re
import collections



## the word counting function 
def count_words(data,Ignore,show_list):

    counter_dict=collections.Counter()
    total_count=0
    
    ## spliting the lines into words, containg letters only
    for line in data:
        line=line.rstrip()
        split_list=re.split("[^a-zA-ZäöüÄÖÜß]",line)

        for word in split_list:
            if len(word) > 0:
              #the ignore case is currently in the for loop - I would be better to put it out - this way the if check is computetd allways -> a bit slower
              #total_count can be ommited by len(split_list)
                if Ignore == True:
                    word=word.lower()
                total_count+=1
                counter_dict[word]+=1
                
    unique=len(counter_dict)
    
    if show_list == True:
        for word in counter_dict:
            print(f"{word}\t{counter_dict[word]}\n")
    else:
        print(f"{unique}/{total_count}")
    return


def main(args):
    ## Assigning new names to input variables
    input_file=args.file 
    Ignore=args.Ignore_case
    show_list=args.countlist
    
    with open (input_file) as file:
        data = file.readlines()
        
    count_words(data,Ignore,show_list)





if __name__ == "__main__":
    import argparse

    # setting obligatory and optional comand line argumnents 
    parser = argparse.ArgumentParser(description='word count in input file uniqe/total word count')
    
    parser.add_argument("file", help="the path to input file")
    parser.add_argument("-l","--countlist",
                        help="shows list of different word counts",
                        action="store_true")
    parser.add_argument("-I","--Ignore_case",help="Ignore case sensitivity",action="store_true")
    args = parser.parse_args()
    main(args)


# -*- coding: utf-8 -*-
"""
Created on Sun Mar  7 18:40:13 2021

@author: Lino Boehler

"""
def main(args):
    input_file=args.file
    with open (input_file) as file:
    
       data = file.read()
       data=data.rstrip()
    
    hello="Hello World!"
    print(f"{hello}\n{data}")
    return


if __name__ == "__main__":
    import sys
    input_file=sys.argv[1]
    main(input_file)




# -*- coding: utf-8 -*-
"""
Created on Sun Mar 21 18:12:01 2021

@author: Lino
"""

import itertools
import numpy as np
import collections






def branch_n_bound(state_nr,cost_dict,path_list,cost_max,city_list,col_names, opt_flag,curent_path,best):
    
    
    
    
    if len(city_list) > 0:
        city_a=city_list.pop(0)
        best=cost_max+1
    else:
        return
    city_l_copy=city_list.copy()
   
    result_list=[]
    curent_paths_list=[]
    best_ls=[]
    for ind in range(len(city_l_copy)):
        
        city_b=city_list[ind]
        curent_pair = (city_a,city_b)
        
        
        if cost_dict[curent_pair] < cost_max:
            #city_l_copy.remove(city_b)
            #curent_cost=cost_dict[curent_pair]
            if len(curent_path) <= state_nr/2-1:
                
                copy_path=curent_path.copy()
                copy_path.append(curent_pair)
                
                if len(copy_path) == state_nr/2:
                    if opt_flag==True:
                        curent_cost=0
                        for i in  copy_path:
                            curent_cost+=cost_dict[i]
                        if curent_cost < best:
                            best = curent_cost
                            best_path=copy_path
                            path_list=[best,best_path]
                            print(f"new best Partition: \n {best_path} \n cost: {best}")
                    curent_paths_list.append(copy_path)
                    
                    
                else:
                    
                    branch_n_bound(state_nr,cost_dict,path_list,cost_max,city_l_copy,col_names, opt_flag,copy_path,best)
            else:
                if len(curent_path) <= state_nr/2:
                    curent_paths_list.append(curent_path)
            
             
            
            #print(path_list)
             
        else:
            continue
   
        
    if len(curent_paths_list) > 0:
       path_list.extend(curent_paths_list)
   
    
    return path_list
    

def main(args):
    ## Assigning new names to input variables
    path=args.file
    opt_flag=args.optimize
    
    #opt_flag=True
    
    #path="Administration-test1.in.txt"
    
    with open(path) as f:
        data=f.readlines()
    
    data=[i.rstrip() for i in data]
    data=[i.split() for i in data]
    
    data_info=data.pop(0)
    col_names=data.pop(0)
    
    data_df=np.array(data)
    state_nr=int(data_info[0])
    cost_max=int(data_info[1])
    
    # generating all possibel capital paires
    capital_combis=itertools.combinations(col_names,2)
    capital_combis=list(capital_combis)
    
    
    cost_dict=collections.defaultdict()
    index=0
    for i in range(0,len(col_names)):
        for j in range (i+1,len(col_names) ):
            
            cost_dict[capital_combis[index]]=int(data_df[i][j])
            
            index+=1
    
    
    path_list=[]
    city_list=col_names
    curent_path=[]
    best=int()

    results=branch_n_bound(state_nr,cost_dict,path_list,cost_max,city_list,col_names, opt_flag,curent_path,best)
    rs=[]
    if opt_flag == False:
        for lst in results:
            path=[''.join(tups) for tups in lst] 
            conc=" ".join(path)
            print(conc)
            
            rs.append(conc)
        


#import argparse

if __name__ == "__main__":
    import argparse
    # setting obligatory and optional comand line argumnents 
    parser = argparse.ArgumentParser(description='')
    
    parser.add_argument("file", help="the path to input file")
    parser.add_argument("-o","--optimize",
                        help="prints optimal solution, instead of  enumeration",
                        action="store_true")
    
    args = parser.parse_args()
    main(args)

        
            



from setuptools import setup, find_packages

setup(
   name='Lino_Boehler_A5',
   version='0.1.0',
   author='Lino Boehler',
   author_email='lino.boehler@gmx.net',
   #packages=["A0","A1","A2","A3","A4"],
   #scripts=['bin/script1','bin/script2'],
   license="MIT-License",
	 packages=find_packages(),
   description='First Package include all Exercises so far',
   long_description=open('README.txt').read(),
   install_requires=[
       "numpy"    
       
   ],
 		
)

